@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Help file control                                                    */
@@ -23,13 +23,16 @@
   Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifndef _HELP_H
+#define _HELP_H
+
 typedef union
 {
 	off_t file_offset;
 	char *mem_offset;
 } help_off_t;
 
-typedef struct
+typedef struct help_t
 {
 	int fd;
 	time_t mtime;
@@ -39,6 +42,7 @@ typedef struct
 	void *next;
 } help_t;
 
-help_t *help_init( irc_t *irc );
-char *help_get( irc_t *irc, char *string );
+help_t *help_init( help_t **help );
+char *help_get( help_t **help, char *string );
 
+#endif
